package com.nwalsh.iss5691;

import net.sf.saxon.s9api.*;

import java.io.File;

public class Main {
    public static void main(String[] args) {
        Main main = new Main();
        try {
            main.run(args);
        } catch (Exception ex) {
            System.err.println("WAT? " + ex.getMessage());
            ex.printStackTrace();
        }
    }

    private void run(String[] args) throws SaxonApiException {
        Processor processor = new Processor(false);
        DocumentBuilder builder = processor.newDocumentBuilder();
        XdmNode main = builder.build(new File("main.xml"));
        XdmNode module = builder.build(new File("module.xml"));

        TreeWriter tree = new TreeWriter(processor);
        tree.startDocument(main.getBaseURI());

        XdmSequenceIterator<XdmNode> iter = main.axisIterator(Axis.CHILD);
        XdmNode root = null;
        while (iter.hasNext()) {
            XdmNode node = iter.next();
            if (XdmNodeKind.ELEMENT.equals(node.getNodeKind())) {
                root = node;
                break;
            }
        }

        assert root != null;

        iter = root.axisIterator(Axis.CHILD);
        while (iter.hasNext()) {
            XdmNode node = iter.next();
            if ("include".equals(node.getNodeName().getLocalName())) {
                tree.addSubtree(module);
            } else {
                tree.addSubtree(node);
            }
        }
        tree.endDocument();

        showtree(tree.getResult(), "");
    }

    private void showtree(XdmNode node, String indent) {
        XdmSequenceIterator<XdmNode> iter;

        switch (node.getNodeKind()) {
            case DOCUMENT:
                iter = node.axisIterator(Axis.CHILD);
                while (iter.hasNext()) {
                    showtree(iter.next(), indent);
                }
                break;

            case PROCESSING_INSTRUCTION:
                System.err.printf("%s%s: %s: %s%n", indent, node.getNodeName().getLocalName(), node.getBaseURI(), node.getUnderlyingNode().getSystemId());
                break;

            case ELEMENT:
                System.err.printf("%s%s: %s: %s%n", indent, node.getNodeName().getLocalName(), node.getBaseURI(), node.getUnderlyingNode().getSystemId());
                iter = node.axisIterator(Axis.CHILD);
                while (iter.hasNext()) {
                    showtree(iter.next(), "  " + indent);
                }

            default:
                break;
        }
    }
}
